@@ -2,23 +2,32 @@ import glob
 import json
 import os
 import shutil
-from importlib import import_module
-from inspect import signature
 from os import listdir
 from os.path import isfile, join
 from pathlib import Path, PurePath
 from re import search
-from typing import List, Optional, Match, Dict, Any, Tuple, Union, Callable
+from typing import List, Optional, Match, Dict, Any, Tuple, Union, Callable, Type
 
+from pyspark.ml import Transformer
 from pyspark.sql import SparkSession, DataFrame
 from pyspark.sql.catalog import Table
 from pyspark.sql.types import StructType
-from spark_data_frame_comparer.spark_data_frame_comparer import assert_compare_data_frames
-from spark_data_frame_comparer.spark_data_frame_comparer_exception import SparkDataFrameComparerException, ExceptionType
+from spark_data_frame_comparer.spark_data_frame_comparer import (
+    assert_compare_data_frames,
+)
+from spark_data_frame_comparer.spark_data_frame_comparer_exception import (
+    SparkDataFrameComparerException,
+    ExceptionType,
+)
 from spark_pipeline_framework.progress_logger.progress_logger import ProgressLogger
-from spark_pipeline_framework.utilities.json_to_jsonl_converter import convert_json_to_jsonl
+from spark_pipeline_framework.utilities.class_helpers import ClassHelpers
+from spark_pipeline_framework.utilities.json_to_jsonl_converter import (
+    convert_json_to_jsonl,
+)
 
-from spark_pipeline_framework_testing.testing_exception import SparkPipelineFrameworkTestingException
+from spark_pipeline_framework_testing.testing_exception import (
+    SparkPipelineFrameworkTestingException,
+)
 
 
 class SparkPipelineFrameworkTestRunner:
@@ -29,9 +38,11 @@ class SparkPipelineFrameworkTestRunner:
         spark_session: SparkSession,
         folder_path: Path,
         parameters: Optional[Dict[str, Any]] = None,
-        func_path_modifier: Optional[Callable[[Union[Path, str]],
-                                              Union[Path, str]]] = None,
-        temp_folder: Optional[Path] = None
+        func_path_modifier: Optional[
+            Callable[[Union[Path, str]], Union[Path, str]]
+        ] = None,
+        temp_folder: Optional[Path] = None,
+        transformer_type: Optional[Type[Transformer]] = None,
     ) -> None:
         """
         Tests Spark Transformers without writing any code
@@ -52,6 +63,7 @@ class SparkPipelineFrameworkTestRunner:
         :param parameters: (Optional) any parameters to pass to the transformer
         :param func_path_modifier: (Optional) A function that can transform the paths
         :param temp_folder: folder to use for temporary files.  Any existing files in this folder will be deleted.
+        :param transformer_type: (Optional) the transformer to run
         :return: Throws SparkPipelineFrameworkTestingException if there are mismatches between
                     expected output files and actual output files.  The `exceptions` list in
                     SparkPipelineFrameworkTestingException holds all the mismatch exceptions
@@ -60,9 +72,7 @@ class SparkPipelineFrameworkTestRunner:
             parameters = {}
 
         # iterate through sub_folders trying to find folders that contain input and output folders
-        testable_folder_list: List[str] = get_testable_folders(
-            folder_path=folder_path
-        )
+        testable_folder_list: List[str] = get_testable_folders(folder_path=folder_path)
 
         # first clear any stuff in SparkSession
         clean_spark_session(session=spark_session)
@@ -72,11 +82,9 @@ class SparkPipelineFrameworkTestRunner:
         for testable_folder in testable_folder_list:
             input_folder: Path = Path(testable_folder).joinpath("input")
             input_files: List[str] = [
-                f for f in listdir(input_folder)
-                if isfile(join(input_folder, f))
+                f for f in listdir(input_folder) if isfile(join(input_folder, f))
             ]
-            input_schema_folder = Path(testable_folder
-                                       ).joinpath("input_schema")
+            input_schema_folder = Path(testable_folder).joinpath("input_schema")
 
             print(f"Running test in folder: {testable_folder}...")
 
@@ -87,17 +95,14 @@ class SparkPipelineFrameworkTestRunner:
                     spark_session=spark_session,
                     input_file=input_file,
                     input_folder=input_folder,
-                    input_schema_folder=input_schema_folder
+                    input_schema_folder=input_schema_folder,
                 )
 
             # write out any input schemas
-            input_tables: List[Table] = spark_session.catalog.listTables(
-                "default"
-            )
+            input_tables: List[Table] = spark_session.catalog.listTables("default")
 
             input_table_names: List[str] = [
-                t.name for t in input_tables
-                if not t.name.startswith("expected_")
+                t.name for t in input_tables if not t.name.startswith("expected_")
             ]
             table_name: str
             for table_name in input_table_names:
@@ -107,12 +112,13 @@ class SparkPipelineFrameworkTestRunner:
                 SparkPipelineFrameworkTestRunner.write_schema_to_output(
                     spark_session=spark_session,
                     view_name=table_name,
-                    schema_folder=input_schema_folder
+                    schema_folder=input_schema_folder,
                 )
 
             # read parameters.json if it exists
-            parameters_json_file: Path = Path(testable_folder
-                                              ).joinpath("parameters.json")
+            parameters_json_file: Path = Path(testable_folder).joinpath(
+                "parameters.json"
+            )
             if os.path.exists(parameters_json_file):
                 with open(parameters_json_file, "r") as file:
                     parameters = json.loads(file.read())
@@ -125,29 +131,29 @@ class SparkPipelineFrameworkTestRunner:
                 parameters["view"] = destination_view_name
 
             # find name of transformer
-            search_result: Optional[Match[str]
-                                    ] = search(r'/library/', testable_folder)
+            search_result: Optional[Match[str]] = search(r"/library/", testable_folder)
             if search_result:
                 SparkPipelineFrameworkTestRunner.run_transformer(
                     spark_session=spark_session,
                     parameters=parameters,
                     search_result=search_result,
-                    testable_folder=testable_folder
+                    testable_folder=testable_folder,
+                    transformer_type=transformer_type,
                 )
 
             # write out any missing schemas
-            output_tables: List[Table] = spark_session.catalog.listTables(
-                "default"
-            )
+            output_tables: List[Table] = spark_session.catalog.listTables("default")
 
-            output_schema_folder: Path = Path(testable_folder
-                                              ).joinpath("output_schema")
+            output_schema_folder: Path = Path(testable_folder).joinpath("output_schema")
             tables_for_writing_schema: List[str] = [
-                t.name for t in output_tables
+                t.name
+                for t in output_tables
                 if not t.name.startswith("expected_")
                 and t.name not in input_table_names
             ]
-            if "output" in tables_for_writing_schema:  # if there is an output table then ignore other input_tables
+            if (
+                "output" in tables_for_writing_schema
+            ):  # if there is an output table then ignore other input_tables
                 tables_for_writing_schema = ["output"]
             for table_name in tables_for_writing_schema:
                 if not os.path.exists(output_schema_folder):
@@ -156,7 +162,7 @@ class SparkPipelineFrameworkTestRunner:
                 SparkPipelineFrameworkTestRunner.write_schema_to_output(
                     spark_session=spark_session,
                     view_name=table_name,
-                    schema_folder=output_schema_folder
+                    schema_folder=output_schema_folder,
                 )
 
             # for each file in output folder, loading into a view in Spark (prepend with "expected_")
@@ -164,8 +170,7 @@ class SparkPipelineFrameworkTestRunner:
             if not os.path.exists(output_folder):
                 os.mkdir(output_folder)
             output_files = [
-                f for f in listdir(output_folder)
-                if isfile(join(output_folder, f))
+                f for f in listdir(output_folder) if isfile(join(output_folder, f))
             ]
             views_found: List[str] = []
             if not temp_folder:
@@ -178,36 +183,44 @@ class SparkPipelineFrameworkTestRunner:
             for output_file in output_files:
                 found_output_file: bool
                 data_frame_exception: Optional[SparkDataFrameComparerException]
-                found_output_file, data_frame_exception = SparkPipelineFrameworkTestRunner.process_output_file(
+                (
+                    found_output_file,
+                    data_frame_exception,
+                ) = SparkPipelineFrameworkTestRunner.process_output_file(
                     spark_session=spark_session,
                     output_file=output_file,
                     output_folder=output_folder,
                     output_schema_folder=output_schema_folder,
                     temp_folder=temp_folder.joinpath("result"),
-                    func_path_modifier=func_path_modifier
+                    func_path_modifier=func_path_modifier,
                 )
                 if found_output_file:
                     views_found.append(
-                        SparkPipelineFrameworkTestRunner.
-                        get_view_name_from_file_path(output_file).lower()
+                        SparkPipelineFrameworkTestRunner.get_view_name_from_file_path(
+                            output_file
+                        ).lower()
                     )
                     if data_frame_exception:
                         data_frame_exceptions.append(data_frame_exception)
 
             # write out any missing output files
             table_names_to_write_to_output: List[str] = [
-                t.name for t in output_tables
-                if t.name.lower() not in views_found and not t.name.
-                startswith("expected_") and t.name not in input_table_names
+                t.name
+                for t in output_tables
+                if t.name.lower() not in views_found
+                and not t.name.startswith("expected_")
+                and t.name not in input_table_names
             ]
-            if "output" in table_names_to_write_to_output:  # if there is an output table then ignore other input_tables
+            if (
+                "output" in table_names_to_write_to_output
+            ):  # if there is an output table then ignore other input_tables
                 table_names_to_write_to_output = ["output"]
             for table_name in table_names_to_write_to_output:
                 SparkPipelineFrameworkTestRunner.write_table_to_output(
                     spark_session=spark_session,
                     view_name=table_name,
                     output_folder=output_folder,
-                    temp_folder=temp_folder.joinpath("result")
+                    temp_folder=temp_folder.joinpath("result"),
                 )
 
             clean_spark_session(session=spark_session)
@@ -219,54 +232,43 @@ class SparkPipelineFrameworkTestRunner:
 
     @staticmethod
     def run_transformer(
-        spark_session: SparkSession, parameters: Optional[Dict[str, Any]],
-        search_result: Match[str], testable_folder: str
+        spark_session: SparkSession,
+        parameters: Optional[Dict[str, Any]],
+        search_result: Match[str],
+        testable_folder: str,
+        transformer_type: Optional[Type[Transformer]],
     ) -> None:
-        # get name of transformer file
-        transformer_file_name = testable_folder[search_result.end():].replace(
-            '/', '_'
-        )
-        # find parent folder of transformer file
-        lib_path = testable_folder[search_result.start() +
-                                   1:].replace('/', '.')
-        # load the transformer file (i.e., module)
-        module = import_module(lib_path + "." + transformer_file_name)
-        md = module.__dict__
-        # find the first class in that module (we assume the first class is the Transformer class)
-        my_class = [
-            md[c] for c in md if
-            (isinstance(md[c], type) and md[c].__module__ == module.__name__)
-        ][0]
-        # find the signature of the __init__ method
-        my_class_signature = signature(my_class.__init__)
-        my_class_args = [
-            param.name for param in my_class_signature.parameters.values()
-            if param.name != 'self'
-        ]
+        my_class: Type[Transformer]
+        if transformer_type:
+            my_class = transformer_type
+        else:
+            # get name of transformer file
+            search_result_end = search_result.end()
+            transformer_file_name: str = testable_folder[search_result_end:].replace(
+                "/", "_"
+            )
+            # find parent folder of transformer file
+            search_result_start_ = search_result.start() + 1
+            lib_path: str = testable_folder[search_result_start_:].replace("/", ".")
+            # load the transformer file (i.e., module)
+            full_reference = lib_path + "." + transformer_file_name
+            my_class = ClassHelpers.get_first_class_in_file(full_reference)
+
         with ProgressLogger() as progress_logger:
             # now figure out the class_parameters to use when instantiating the class
             class_parameters: Dict[str, Any] = {
                 "parameters": parameters or {},
-                'progress_logger': progress_logger
+                "progress_logger": progress_logger,
             }
-            # instantiate the class passing in the parameters + progress_logger
-            if len(my_class_args) > 0 and len(class_parameters) > 0:
-                my_instance = my_class(
-                    **{
-                        k: v
-                        for k, v in class_parameters.items()
-                        if k in my_class_args
-                    }
-                )
-            else:
-                my_instance = my_class()
+            my_instance: Transformer = ClassHelpers.instantiate_class_with_parameters(
+                class_parameters=class_parameters, my_class=my_class
+            )
             # now call transform
             schema = StructType([])
             # create an empty dataframe to pass into transform()
             df: DataFrame = spark_session.createDataFrame(
                 spark_session.sparkContext.emptyRDD(), schema
             )
-
             my_instance.transform(df)
 
     @staticmethod
@@ -275,34 +277,31 @@ class SparkPipelineFrameworkTestRunner:
         output_file: str,
         output_folder: Path,
         output_schema_folder: Path,
-        func_path_modifier: Optional[Callable[[Union[Path, str]], Union[Path,
-                                                                        str]]],
+        func_path_modifier: Optional[Callable[[Union[Path, str]], Union[Path, str]]],
         temp_folder: Optional[Union[Path, str]] = None,
     ) -> Tuple[bool, Optional[SparkDataFrameComparerException]]:
         data_frame_exception: Optional[SparkDataFrameComparerException] = None
-        file_extension: str = SparkPipelineFrameworkTestRunner.get_file_extension_from_file_path(
-            output_file
+        file_extension: str = (
+            SparkPipelineFrameworkTestRunner.get_file_extension_from_file_path(
+                output_file
+            )
         )
         view_name: str = SparkPipelineFrameworkTestRunner.get_view_name_from_file_path(
             output_file
         )
-        if file_extension.lower() not in [
-            ".csv", ".json", ".jsonl", ".parquet"
-        ]:
+        if file_extension.lower() not in [".csv", ".json", ".jsonl", ".parquet"]:
             return True, data_frame_exception
         result_df: DataFrame = spark_session.table(view_name)
 
         found_output_file: bool = False
         output_file_path = os.path.join(output_folder, output_file)
-        result_path: Optional[Path] = Path(temp_folder).joinpath(
-            f"{view_name}"
-        ) if temp_folder else None
+        result_path: Optional[Path] = (
+            Path(temp_folder).joinpath(f"{view_name}") if temp_folder else None
+        )
         result_file: Optional[Path] = None
         output_schema_file: Optional[str] = None
         if file_extension.lower() == ".csv":
-            output_schema_file = os.path.join(
-                output_schema_folder, f"{view_name}.json"
-            )
+            output_schema_file = os.path.join(output_schema_folder, f"{view_name}.json")
             # if we have an output schema file use it
             if os.path.exists(output_schema_file):
                 with open(output_schema_file) as file:
@@ -312,18 +311,16 @@ class SparkPipelineFrameworkTestRunner:
                     f"Reading file {output_file_path} using schema: {output_schema_file}"
                 )
                 spark_session.read.schema(schema).csv(
-                    path=output_file_path, header=True
+                    path=output_file_path, header=True, comment="#", emptyValue=None
                 ).createOrReplaceTempView(f"expected_{view_name}")
             else:  # if we don't have schema file then load without a schema
                 spark_session.read.csv(
-                    path=output_file_path, header=True, comment="#"
+                    path=output_file_path, header=True, comment="#", emptyValue=None
                 ).createOrReplaceTempView(f"expected_{view_name}")
 
             # write the result file to temp folder
             if result_path and temp_folder:
-                result_path_for_view: Path = result_path.joinpath(
-                    f"{view_name}.csv"
-                )
+                result_path_for_view: Path = result_path.joinpath(f"{view_name}.csv")
                 result_df.coalesce(1).write.csv(
                     path=str(result_path_for_view), header=True
                 )
@@ -331,15 +328,12 @@ class SparkPipelineFrameworkTestRunner:
                 SparkPipelineFrameworkTestRunner.combine_spark_csv_files_to_one_file(
                     source_folder=result_path_for_view,
                     destination_file=result_file,
-                    file_extension="csv"
+                    file_extension="csv",
                 )
 
             found_output_file = True
-        elif file_extension.lower() == ".jsonl" or file_extension.lower(
-        ) == ".json":
-            output_schema_file = os.path.join(
-                output_schema_folder, output_file
-            )
+        elif file_extension.lower() == ".jsonl" or file_extension.lower() == ".json":
+            output_schema_file = os.path.join(output_schema_folder, output_file)
             if os.path.exists(output_schema_file):
                 with open(output_schema_file) as file:
                     schema_json = json.loads(file.read())
@@ -347,49 +341,43 @@ class SparkPipelineFrameworkTestRunner:
                 print(
                     f"Reading file {output_file_path} using schema: {output_schema_file}"
                 )
-                spark_session.read.schema(schema).option(
-                    "multiLine", True
-                ).json(path=output_file_path
-                       ).createOrReplaceTempView(f"expected_{view_name}")
+                spark_session.read.schema(schema).option("multiLine", True).json(
+                    path=output_file_path
+                ).createOrReplaceTempView(f"expected_{view_name}")
             else:
                 spark_session.read.option("multiLine", True).json(
                     path=output_file_path
                 ).createOrReplaceTempView(f"expected_{view_name}")
             # write result to temp folder for comparison
             if result_path and temp_folder:
-                result_path_for_view = result_path.joinpath(
-                    f"{view_name}.json"
-                )
-                result_df.coalesce(1).write.json(
-                    path=str(result_path_for_view)
-                )
+                result_path_for_view = result_path.joinpath(f"{view_name}.json")
+                result_df.coalesce(1).write.json(path=str(result_path_for_view))
                 result_file = Path(temp_folder).joinpath(f"{view_name}.json")
                 SparkPipelineFrameworkTestRunner.combine_spark_json_files_to_one_file(
                     source_folder=result_path_for_view,
                     destination_file=result_file,
-                    file_extension="json"
+                    file_extension="json",
                 )
             found_output_file = True
         elif file_extension.lower() == ".parquet":
-            spark_session.read.parquet(
-                path=output_file_path
-            ).createOrReplaceTempView(f"expected_{view_name}")
+            spark_session.read.parquet(path=output_file_path).createOrReplaceTempView(
+                f"expected_{view_name}"
+            )
             found_output_file = True
         if found_output_file:
             # Do a data frame compare on each view
-            print(
-                f"Comparing with view:[view_name= with view:[expected_{view_name}]"
-            )
+            print(f"Comparing with view:[view_name= with view:[expected_{view_name}]")
             try:
                 # drop any corrupted column
                 assert_compare_data_frames(
-                    expected_df=spark_session.table(f"expected_{view_name}"
-                                                    ).drop("_corrupt_record"),
+                    expected_df=spark_session.table(f"expected_{view_name}").drop(
+                        "_corrupt_record"
+                    ),
                     result_df=result_df,
                     result_path=result_file,
                     expected_path=output_file_path,
                     temp_folder=temp_folder,
-                    func_path_modifier=func_path_modifier
+                    func_path_modifier=func_path_modifier,
                 )
             except SparkDataFrameComparerException as e:
                 data_frame_exception = e
@@ -397,19 +385,24 @@ class SparkPipelineFrameworkTestRunner:
                 if e.exception_type == ExceptionType.SchemaMismatch:
                     if temp_folder and output_schema_file:
                         # write the new schema to temp folder
-                        result_schema_path = SparkPipelineFrameworkTestRunner.write_schema_to_output(
-                            spark_session=spark_session,
-                            view_name=view_name,
-                            schema_folder=Path(temp_folder).joinpath(
-                                "schemas"
-                            ).joinpath("result").joinpath(view_name)
+                        result_schema_path = (
+                            SparkPipelineFrameworkTestRunner.write_schema_to_output(
+                                spark_session=spark_session,
+                                view_name=view_name,
+                                schema_folder=Path(temp_folder)
+                                .joinpath("schemas")
+                                .joinpath("result")
+                                .joinpath(view_name),
+                            )
                         )
-                        e.compare_path = SparkPipelineFrameworkTestRunner.get_compare_path(
-                            result_path=result_schema_path,
-                            expected_path=Path(output_schema_file),
-                            temp_folder=temp_folder,
-                            func_path_modifier=func_path_modifier,
-                            type_="schema"
+                        e.compare_path = (
+                            SparkPipelineFrameworkTestRunner.get_compare_path(
+                                result_path=result_schema_path,
+                                expected_path=Path(output_schema_file),
+                                temp_folder=temp_folder,
+                                func_path_modifier=func_path_modifier,
+                                type_="schema",
+                            )
                         )
                         if func_path_modifier and e.compare_path:
                             e.compare_path = func_path_modifier(e.compare_path)
@@ -418,10 +411,11 @@ class SparkPipelineFrameworkTestRunner:
 
     @staticmethod
     def get_compare_path(
-        result_path: Optional[Path], expected_path: Optional[Path],
+        result_path: Optional[Path],
+        expected_path: Optional[Path],
         temp_folder: Optional[Union[Path, str]],
-        func_path_modifier: Optional[Callable[[Union[Path, str]],
-                                              Union[Path, str]]], type_: str
+        func_path_modifier: Optional[Callable[[Union[Path, str]], Union[Path, str]]],
+        type_: str,
     ) -> Optional[Path]:
         compare_sh_path: Optional[Path] = None
         if expected_path and result_path and temp_folder:
@@ -442,50 +436,55 @@ class SparkPipelineFrameworkTestRunner:
 
     @staticmethod
     def process_input_file(
-        spark_session: SparkSession, input_file: str, input_folder: Path,
-        input_schema_folder: Path
+        spark_session: SparkSession,
+        input_file: str,
+        input_folder: Path,
+        input_schema_folder: Path,
     ) -> None:
-        file_extension: str = SparkPipelineFrameworkTestRunner.get_file_extension_from_file_path(
-            input_file
+        file_extension: str = (
+            SparkPipelineFrameworkTestRunner.get_file_extension_from_file_path(
+                input_file
+            )
         )
         view_name: str = SparkPipelineFrameworkTestRunner.get_view_name_from_file_path(
             input_file
         )
         if file_extension.lower() == ".csv":
             input_file_path = os.path.join(input_folder, input_file)
-            input_schema_file = os.path.join(
-                input_schema_folder, f"{view_name}.json"
-            )
+            input_schema_file = os.path.join(input_schema_folder, f"{view_name}.json")
             if os.path.exists(input_schema_file):
                 with open(input_schema_file) as file:
                     schema_json: str = json.loads(file.read())
                 schema = StructType.fromJson(schema_json)
-                print(
-                    f"Reading file {input_file} using schema: {input_schema_file}"
-                )
+                print(f"Reading file {input_file} using schema: {input_schema_file}")
                 spark_session.read.schema(schema).csv(
                     path=input_file_path,
                     header=True,
                     comment="#",
                     emptyValue=None,
-                ).limit(SparkPipelineFrameworkTestRunner.row_limit
-                        ).createOrReplaceTempView(view_name)
-                assert "_corrupt_record" not in spark_session.table(
+                ).limit(
+                    SparkPipelineFrameworkTestRunner.row_limit
+                ).createOrReplaceTempView(
                     view_name
-                ).columns, input_file_path
+                )
+                assert (
+                    "_corrupt_record" not in spark_session.table(view_name).columns
+                ), input_file_path
             else:
                 spark_session.read.csv(
                     path=input_file_path,
                     header=True,
                     comment="#",
                     emptyValue=None,
-                ).limit(SparkPipelineFrameworkTestRunner.row_limit
-                        ).createOrReplaceTempView(view_name)
-                assert "_corrupt_record" not in spark_session.table(
+                ).limit(
+                    SparkPipelineFrameworkTestRunner.row_limit
+                ).createOrReplaceTempView(
                     view_name
-                ).columns, input_file_path
-        elif file_extension.lower() == ".jsonl" or file_extension.lower(
-        ) == ".json":
+                )
+                assert (
+                    "_corrupt_record" not in spark_session.table(view_name).columns
+                ), input_file_path
+        elif file_extension.lower() == ".jsonl" or file_extension.lower() == ".json":
             input_file_path = os.path.join(input_folder, input_file)
             # create json_input_folder if it does not exist
             json_input_folder = os.path.join(input_folder, "..", "input_jsonl")
@@ -494,8 +493,7 @@ class SparkPipelineFrameworkTestRunner:
             jsonl_input_file_path = os.path.join(json_input_folder, input_file)
             # convert file to jsonl if needed
             convert_json_to_jsonl(
-                src_file=Path(input_file_path),
-                dst_file=Path(jsonl_input_file_path)
+                src_file=Path(input_file_path), dst_file=Path(jsonl_input_file_path)
             )
             # find schema file
             input_schema_file = os.path.join(input_schema_folder, input_file)
@@ -504,31 +502,32 @@ class SparkPipelineFrameworkTestRunner:
                 with open(input_schema_file) as file:
                     schema_json = json.loads(file.read())
                 schema = StructType.fromJson(schema_json)
-                print(
-                    f"Reading file {input_file} using schema: {input_schema_file}"
-                )
+                print(f"Reading file {input_file} using schema: {input_schema_file}")
                 spark_session.read.schema(schema).json(
                     path=jsonl_input_file_path
-                ).limit(SparkPipelineFrameworkTestRunner.row_limit
-                        ).createOrReplaceTempView(view_name)
-                assert "_corrupt_record" not in spark_session.table(
+                ).limit(
+                    SparkPipelineFrameworkTestRunner.row_limit
+                ).createOrReplaceTempView(
                     view_name
-                ).columns, input_file_path
+                )
+                assert (
+                    "_corrupt_record" not in spark_session.table(view_name).columns
+                ), input_file_path
             else:  # if no schema found just load the file
                 spark_session.read.json(path=jsonl_input_file_path).limit(
                     SparkPipelineFrameworkTestRunner.row_limit
                 ).createOrReplaceTempView(view_name)
-                assert "_corrupt_record" not in spark_session.table(
-                    view_name
-                ).columns, input_file_path
+                assert (
+                    "_corrupt_record" not in spark_session.table(view_name).columns
+                ), input_file_path
         elif file_extension.lower() == ".parquet":
             input_file_path = os.path.join(input_folder, input_file)
             spark_session.read.parquet(path=input_file_path).limit(
                 SparkPipelineFrameworkTestRunner.row_limit
             ).createOrReplaceTempView(view_name)
-            assert "_corrupt_record" not in spark_session.table(
-                view_name
-            ).columns, input_file_path
+            assert (
+                "_corrupt_record" not in spark_session.table(view_name).columns
+            ), input_file_path
 
     @staticmethod
     def get_view_name_from_file_path(input_file: str) -> str:
@@ -549,19 +548,17 @@ class SparkPipelineFrameworkTestRunner:
         # these type strings can look like 'array<struct<Field:string>>', so we
         # have to check if "array" or "struct" appears in the type string, not
         # just for exact matches
-        return any(
-            [t for t in type_dict.values() if "array" in t or "struct" in t]
-        )
+        return any([t for t in type_dict.values() if "array" in t or "struct" in t])
 
     @staticmethod
     def write_table_to_output(
-        spark_session: SparkSession, view_name: str, output_folder: Path,
-        temp_folder: Path
+        spark_session: SparkSession,
+        view_name: str,
+        output_folder: Path,
+        temp_folder: Path,
     ) -> None:
         df: DataFrame = spark_session.table(view_name)
-        if SparkPipelineFrameworkTestRunner.should_write_dataframe_as_json(
-            df=df
-        ):
+        if SparkPipelineFrameworkTestRunner.should_write_dataframe_as_json(df=df):
             # save as json
             file_path: Path = temp_folder.joinpath(f"{view_name}.json")
             print(f"Writing {file_path}")
@@ -569,28 +566,25 @@ class SparkPipelineFrameworkTestRunner:
             SparkPipelineFrameworkTestRunner.combine_spark_json_files_to_one_file(
                 source_folder=file_path,
                 destination_file=output_folder.joinpath(f"{view_name}.json"),
-                file_extension="json"
+                file_extension="json",
             )
         else:
             # save as csv
             file_path = temp_folder.joinpath(f"{view_name}.csv")
             print(f"Writing {file_path}")
 
-            df.coalesce(1).write.mode("overwrite"
-                                      ).csv(path=str(file_path), header=True)
+            df.coalesce(1).write.mode("overwrite").csv(path=str(file_path), header=True)
             SparkPipelineFrameworkTestRunner.combine_spark_csv_files_to_one_file(
                 source_folder=file_path,
                 destination_file=output_folder.joinpath(f"{view_name}.csv"),
-                file_extension="csv"
+                file_extension="csv",
             )
 
     @staticmethod
     def combine_spark_csv_files_to_one_file(
         source_folder: Path, destination_file: Path, file_extension: str
     ) -> None:
-        file_pattern_to_search: Path = source_folder.joinpath(
-            f"*.{file_extension}"
-        )
+        file_pattern_to_search: Path = source_folder.joinpath(f"*.{file_extension}")
         # find files with that extension in source_folder
         files: List[str] = glob.glob(str(file_pattern_to_search))
         lines: List[str] = []
@@ -606,9 +600,7 @@ class SparkPipelineFrameworkTestRunner:
     def combine_spark_json_files_to_one_file(
         source_folder: Path, destination_file: Path, file_extension: str
     ) -> None:
-        file_pattern_to_search: Path = source_folder.joinpath(
-            f"*.{file_extension}"
-        )
+        file_pattern_to_search: Path = source_folder.joinpath(f"*.{file_extension}")
         # find files with that extension in source_folder
         files: List[str] = glob.glob(str(file_pattern_to_search))
         # now copy the first file to the destination
@@ -638,10 +630,10 @@ class SparkPipelineFrameworkTestRunner:
                 schema_as_dict: Dict[str, Any] = df.schema.jsonValue()
                 # schema_as_dict: Any = json.loads(s=schema_as_json)
                 # Adding $schema tag enables auto-complete and syntax checking in editors
-                schema_as_dict[
-                    "$schema"
-                ] = "https://raw.githubusercontent.com/imranq2/SparkPipelineFramework.Testing/main/spark_json_schema" \
+                schema_as_dict["$schema"] = (
+                    "https://raw.githubusercontent.com/imranq2/SparkPipelineFramework.Testing/main/spark_json_schema"
                     ".json "
+                )
                 file.write(json.dumps(schema_as_dict, indent=4))
         return schema_file_path
 
@@ -649,7 +641,8 @@ class SparkPipelineFrameworkTestRunner:
 def get_testable_folders(folder_path: Path) -> List[str]:
     folder_list: List[str] = list_folders(folder_path=folder_path)
     testable_folder_list: List[str] = [
-        str(PurePath(folder_path).parent) for folder_path in folder_list
+        str(PurePath(folder_path).parent)
+        for folder_path in folder_list
         if PurePath(folder_path).name == "input"
     ]
     return testable_folder_list
